/*
 * interaction.js
 * Copyright  2007 Tommi Rautava
 * 
 * This file is part of Popomungo.
 *
 * Popomungo is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Popomungo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * pm_Interaction
 * 
 * 1 Greet
 * 2 Ask for autograph
 * 3 Talk to
 * 4 Tell joke
 * 5 Tease
 * 6 Google-Google
 * 7 Buy a drink
 * 8 Hug
 * 9 Kiss
 * 10 Kiss passionately
 * 11 Make Love
 * 12 Tickle
 * 13 5 minute quickie
 * 14 Compliment
 * 15 Insult
 * 16 Slap
 * 17
 * 18 Play with
 * 19
 * 20
 * 21 Sing to
 * 22
 * 23
 * 24
 * 25
 * 26
 * 27 
 * 28 End relationship
 * 29 Seek apprenticeship
 * 30 
 * 31 Interrogate
 * 32 Give first aid
 * 33 Do funny magic
 * 34 Have profound discussion
 * 35 Ask for a dance
 */
var pm_Interaction = {
	
	DUMMY_OPTION_CLASS: 'popomungo_dummyOption',
	
	SHOW_HIDDEN_OPTIONS_BUTTON_ID: 'popomungo_showOptions',
	
	HIDDEN_OPTION_CLASS: 'popomungo_hiddenOption',
	
	HIDDEN_OPTION_VISIBLE_CLASS: 'popomungo_hiddenOptionVisible',

	HIDDEN_OPTION_VISIBLE_HL_CLASS: 'popomungo_hiddenOptionVisibleHL',

	OPTION_ALERT_CLASS: 'popomungo_optionAlert',
	
	
	_getSelectInteractionElement: function _getSelectInteractionElement(doc) {
		try {
			return doc.getElementsByName('InteractionTypeID').item(0);
		}
		catch (err) {
			pm_Logger.logError(err);
		}
		
		return null;
	},
	
	
	_addEmptyItem: function _addEmptyItem(doc, aSelectElem) {
		//pm_Logger.debug("aSelectElem="+ aSelectElem);

		/* No interaction:
		 * <select name="InteractionTypeID">
		 *     <option value="">------------</option>
		 * </select>
		 */
		
		var opt1 = doc.createElement('option');
		opt1.value = '';
		opt1.appendChild(doc.createTextNode('------------'));
		opt1.className = this.DUMMY_OPTION_CLASS;
		
		aSelectElem.appendChild(opt1);
	},


	/* Move or remove certain interaction options to the end of the selection list.
	 * 
	 * @param doc
	 */
	sortInteractionChoises: function sortInteractionChoises(doc) {
		try {
			if (!pm_Prefs.isEnabled(pm_PrefKeys.INTERACTION_FEATURES_ENABLED)) {
				return;
			}
			
			var selectInteraction = this._getSelectInteractionElement(doc);
			
			if (!selectInteraction) {
				return;
			}
			
			// Loop through all items.
			optionsLength = selectInteraction.length;
	
			pm_Logger.debug("selectInteraction="+ selectInteraction + ", .length="+ selectInteraction.length);
	
			var hiddenOptions = 0;
	
			var n = 0;		
			for (var i = 0; i < optionsLength; i++) {
				var optionNode = selectInteraction.item(n);
				
				if (!optionNode.value) {
					pm_Logger.debug('skipped item value='+ optionNode.value);
					continue;
				}
				
				var optionValue = parseInt(optionNode.value);
	
				var optionPrefName;
				var colorizePrefName;
	
				optionPrefName = pm_PrefKeys.INTERACTION_OPTION_COMMAND.replace(/<n>/, optionValue);
				colorizePrefName = pm_PrefKeys.INTERACTION_OPTION_HIGHLIGHT.replace(/<n>/, optionValue);
	
				var optionCommand = pm_Prefs.getSetting(optionPrefName, pm_DefaultPrefs.INTERACTION_OPTION_COMMAND);
				var colorizeAction = pm_Prefs.getSetting(colorizePrefName, pm_DefaultPrefs.INTERACTION_OPTION_HIGHLIGHT);
	
				pm_Logger.debug("optionValue="+ optionValue +
					", optionCommand="+ optionCommand +
					", colorizeAction="+ colorizeAction);
	
				if (colorizeAction) {
					optionNode.className = optionNode.className ? 
						' '+ this.OPTION_ALERT_CLASS : this.OPTION_ALERT_CLASS;
				}
	
				switch (optionCommand) {
					case 0: // Do nothing
						pm_Logger.debug("Keep option "+ optionValue);
						n++;
						break;
					case 1: // Move option to the end of the list
						pm_Logger.debug("Move option "+ optionValue);
						selectInteraction.remove(n);
						selectInteraction.appendChild(optionNode);
						break;
					case 2: // Hide option from the list
						pm_Logger.debug("Hide option "+ optionValue);
						selectInteraction.remove(n);
						selectInteraction.appendChild(optionNode);
						optionNode.className = optionNode.className ? 
							' '+ this.HIDDEN_OPTION_CLASS : this.HIDDEN_OPTION_CLASS;
						hiddenOptions++;
						break;
					case undefined:
						pm_Logger.debug("No command assigned");
						n++;
						break;
					default:
						pm_Logger.logError("Unknown command, code="+ optionCommand);
						n++;
				}			
			}
			
			// If the option list is empty, add a dummy item and disable the OK button.
			if ((selectInteraction.length - hiddenOptions) == 0) {
				pm_Logger.debug("Empty list, disable OK button");

				var okButton = selectInteraction.form.elements.namedItem("submit");
				okButton.disabled = true;

				this._addEmptyItem(doc, selectInteraction);
				selectInteraction.selectedIndex = selectInteraction.length - 1;
			}
			else {
				var selectedIndex;
				
				// Find first visible option.
				for (selectedIndex = 0; selectedIndex < selectInteraction.length; selectedIndex++) {
					var optionNode = selectInteraction.item(selectedIndex);
					
					if (optionNode.className) { 
						if (optionNode.className.match(this.HIDDEN_OPTION_CLASS)) {
							pm_Logger.debug(selectedIndex +': hidden');
							continue;
						}
					}

					break;
				} 
				
				selectInteraction.selectedIndex = selectedIndex;
			}
			
			if (hiddenOptions > 0) {
				this.addShowHiddenOptionsButton(doc);	
			}
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	},
	
	
	addShowHiddenOptionsButton:
	function addShowHiddenOptionsButton(doc) {
		try {
			var selectInteraction = this._getSelectInteractionElement(doc);
			var text1 = pm_Strings.getString(pm_StringKeys.INTERACTION_SHOW_HIDDEN_OPTIONS, null);

			var button1 = doc.createElement('input');
			button1.type = "button";
			button1.id = this.SHOW_HIDDEN_OPTIONS_BUTTON_ID;
			button1.name = this.SHOW_HIDDEN_OPTIONS_BUTTON_ID;
			button1.value = text1;
			button1.addEventListener('click', pm_Interaction.showHiddenInteractionOptionsButtonClicked, true);

			var pn = selectInteraction.parentNode;
			pn.appendChild(doc.createElement('br'));
			pn.appendChild(button1);
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	},
	
	
	showHiddenInteractionOptionsButtonClicked:
	function showHiddenInteractionOptionsButtonClicked(aEvent) {
		try {
			var doc = aEvent.currentTarget.ownerDocument;
			var button1 = doc.getElementById(pm_Interaction.SHOW_HIDDEN_OPTIONS_BUTTON_ID);
			button1.disabled = true;

			var selectInteraction = pm_Interaction._getSelectInteractionElement(doc);
			
			if (!selectInteraction) {
				return;
			}
			
			// Loop through all items.
			optionsLength = selectInteraction.length;
	
			pm_Logger.debug("selectInteraction="+ selectInteraction + ", .length="+ selectInteraction.length);
	
			for (var i = optionsLength - 1; i >= 0; i--) {
				var optionNode = selectInteraction.item(i);
				var className;
				
				if (optionNode.className.match(pm_Interaction.HIDDEN_OPTION_CLASS)) {
					var newClass = optionNode.style.backgroundColor ? pm_Interaction.HIDDEN_OPTION_VISIBLE_HL_CLASS : pm_Interaction.HIDDEN_OPTION_VISIBLE_CLASS;
					
					className = optionNode.className.replace(
						pm_Interaction.HIDDEN_OPTION_CLASS,
						newClass);
						
					pm_Logger.debug(i +': className='+ className);
					
					optionNode.className = className;
				}
				else if (optionNode.className.match(pm_Interaction.DUMMY_OPTION_CLASS)) {
					optionNode.style.display = 'none';

					pm_Logger.debug(i +': dummy node is now hidden');
				}	
			}

			selectInteraction.selectedIndex = 0;
			
			var okButton = selectInteraction.form.elements.namedItem("submit");
			okButton.disabled = false;
		}
		catch (err) {
			pm_Logger.logError(err);
		}
	}
};
